/*
*********************************************************************************************************
*                                              EXAMPLE CODE
*
*                             (c) Copyright 2010; Micrium, Inc.; Weston, FL
*
*               All rights reserved.  Protected by international copyright laws.
*               Knowledge of the source code may NOT be used to develop a similar product.
*               Please help us continue to provide the Embedded community with the finest
*               software available.  Your honesty is greatly appreciated.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                            EXAMPLE CODE
*                                    KEIL MCB1700 DEVELOPMENT KIT                                    
*
* Filename      : app.c
* Version       : V1.00
* Programmer(s) : FT
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                             INCLUDE FILES
*********************************************************************************************************
*/

#include <includes.h>
#include "LPC17xx.h"







/*
*********************************************************************************************************
*                                             LOCAL DEFINES
*********************************************************************************************************
*/

#define  APP_ADDR_FIO2PIN_PIN                    ((CPU_ADDR)(0x2009C054))


#define  APP_LED_CPU_PWR_MODE                             1u
#define  APP_LED_HEART_BEAT                               2u


/*
*********************************************************************************************************
*                                            LOCAL VARIABLES
*
* Note(s) : (1) The 'App_DMA_Mem' memory region is used for DMA transfer between main memory and the Port 2
*               pins (LEDs 4 ..8).
*********************************************************************************************************
*/

static  OS_TCB      *App_TaskStartTCBPtr;                       /* Application startup task TCB   pointer.              */
static  CPU_STK     *App_TaskStartStkPtr;                       /* Application startup task stack pointer.              */

                                                                /* DMA memory region. Defines LEDs pattern.             */
static  CPU_INT08U   App_DMA_Mem[10u] = {
   (DEF_BIT_06 |    0u      |    0u      | DEF_BIT_03 | DEF_BIT_02),
   (DEF_BIT_06 | DEF_BIT_05 | DEF_BIT_04 |    0u      | DEF_BIT_02),
   (DEF_BIT_06 |    0u      | DEF_BIT_04 | DEF_BIT_03 | 0u        ),
   (DEF_BIT_06 | DEF_BIT_05 | DEF_BIT_04 |    0u      | 0u        ),
   (DEF_BIT_06 | DEF_BIT_05 | DEF_BIT_04 | DEF_BIT_03 | DEF_BIT_02),
   (DEF_BIT_06 |    0u      | DEF_BIT_04 |    0u      | 0u        ),
   (DEF_BIT_06 | DEF_BIT_05 | 0u         | DEF_BIT_03 | DEF_BIT_02),
   (DEF_BIT_06 | DEF_BIT_05 | DEF_BIT_04 | DEF_BIT_03 | DEF_BIT_02),
   (0u         | DEF_BIT_05 | DEF_BIT_04 | 0u         | 0u        ),
   (0u         | 0u         | 0u         | 0u         | 0u        )
};

static  CPU_BOOLEAN  App_ProbeLED_PwrMode;                       /* Software LEDs to monitor in Probe.                  */
static  CPU_BOOLEAN  App_ProbeLED_HeartBeat;


/*
*********************************************************************************************************
*                                         FUNCTION PROTOTYPES
*********************************************************************************************************
*/

static  void  App_TaskCreate        (void);
static  void  App_ObjCreate         (void);
static  void  App_ObjAlloc          (void);

static  void  App_TaskStart         (void  *p_arg);
static  void  App_Ext0_IntHandler   (void  *p_arg);


/*
*********************************************************************************************************
*                                                main()
*
* Description : This is the standard entry point for C code.  It is assumed that your code will call
*               main() once you have performed all necessary initialization.
*
* Argument(s) : none
*
* Return(s)   : none
*
* Caller(s)   : Startup Code.
*
* Note(s)     : none.
*********************************************************************************************************
*/

int  main (void)
{


    OS_ERR   os_err;

    LPC_GPIO1->FIODIR = 0xFFFFFFFF;               /* LEDs PORT1 are Output */
    LPC_GPIO1->FIOPIN |=  (1<<18);                  /* Turn On  LED */

#if (CPU_CFG_NAME_EN == DEF_ENABLED)
    CPU_ERR  cpu_err;
#endif

    BSP_PreInit();

    CPU_Init();                                                 /* Initialize the uC/CPU services                       */

#if (CPU_CFG_NAME_EN == DEF_ENABLED)
    CPU_NameSet((CPU_CHAR *)CSP_DEV_NAME,
                (CPU_ERR  *)&cpu_err);
#endif

    Mem_Init();                                                 /* Initialize memory management  module                 */

    App_ObjAlloc();                                             /* Allocate objects form the heap.                      */

    OSInit(&os_err);                                            /* Init uC/OS-III.                                      */

    OSTaskCreate((OS_TCB      *)App_TaskStartTCBPtr,            /* Create the start task                                */
                 (CPU_CHAR    *)"Start",
                 (OS_TASK_PTR  )App_TaskStart,
                 (void        *)0,
                 (OS_PRIO      )APP_CFG_TASK_START_PRIO,
                 (CPU_STK     *)App_TaskStartStkPtr,
                 (CPU_STK_SIZE )APP_CFG_TASK_START_STK_SIZE_LIMIT,
                 (CPU_STK_SIZE )APP_CFG_TASK_START_STK_SIZE,
                 (OS_MSG_QTY   )0u,
                 (OS_TICK      )0u,
                 (void        *)0,
                 (OS_OPT       )(OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
                 (OS_ERR      *)&os_err);

    OSStart(&os_err);                                           /* Start multitasking.                                    */

    (void)&os_err;

    return (0);
}


/*
*********************************************************************************************************
*                                          STARTUP TASK
*
* Description : This is an example of a startup task.  As mentioned in the book's text, you MUST
*               initialize the ticker only once multitasking has started.
*
* Arguments   : p_arg   is the argument passed to 'AppTaskStart()' by 'OSTaskCreate()'.
*
* Returns     : none
*
* Notes       : 1) The first line of code is used to prevent a compiler warning because 'p_arg' is not
*                  used.  The compiler should not generate any code for this statement.
*********************************************************************************************************
*/

static  void  App_TaskStart (void *p_arg)
{






    OS_ERR       err;
    CPU_INT08U   time_ctr;
    CSP_DEV_NBR  ch_nbr;


    (void)p_arg;
    (void)App_ProbeLED_PwrMode;
    (void)App_ProbeLED_HeartBeat;



    BSP_PostInit();                                             /* Initialize BSP functions                             */

    OS_CSP_TickInit();                                          /* Initialize the Tick interrupt.                       */

#if (OS_CFG_STAT_TASK_EN > 0u)
    OSStatTaskCPUUsageInit(&err);
#endif


#ifdef CPU_CFG_INT_DIS_MEAS_EN
    CPU_IntDisMeasMaxCurReset();
#endif




    App_SerialInit();                                           /* Initialize serial module.                            */

    App_ProbeInit();                                            /* Initialize Real-Time monitor.                        */

    App_TaskCreate();                                           /* Create Application tasks.                            */

    App_ObjCreate();                                            /* Create Application kernel objects.                   */

    CSP_TmrInit();                                              /* Initialize timer module.                             */
    CSP_DMA_InitExt();                                          /* Initialize DMA module.                               */

                                                                /* Configure Push button I/O pin.                       */
    CSP_GPIO_Cfg((CSP_DEV_NBR    )CSP_GPIO_PORT_NBR_02,         /* ... Port2.                                           */
                 (CSP_GPIO_MSK   )DEF_BIT_10,                   /* ... Pin 2.10.                                        */
                 (CSP_OPT        )CSP_GPIO_DIR_IN,              /* ... Pin configured as input.                         */
                 (CSP_OPT_FLAGS  )CSP_GPIO_FLAG_MODE_NONE,      /* ... Neither  pull-up nor pull-down resistor.         */
                 (CPU_BOOLEAN    )DEF_YES,                      /* ... Interrupt enabled.                               */
                 (CSP_OPT        )CSP_INT_POL_EDGE_NEGATIVE,    /* ... Falling edge.                                    */
                 (CSP_OPT        )CSP_GPIO_FNCT_01);            /* ... Pin configured as EXTINT0.                       */


                                                                /* Allocated a new DMA channel.                         */
    (void)CSP_DMA_CH_AllocExt( CSP_DMA_CH_PRIO_ANY_LOW,
                              &ch_nbr);

    CSP_DMA_CH_CfgExt((CSP_DEV_NBR )ch_nbr,
                      (CSP_OPT     )CSP_DMA_XFER_TYPE_MEM_TO_PER,
                      (CSP_OPT     )0u,
                      (CSP_OPT     )CPU_WORD_SIZE_08,
                      (CSP_OPT     )0u,
                      (CSP_OPT     )CPU_WORD_SIZE_08,
                      (CSP_DEV_NBR )0u,
                      (CSP_DEV_NBR )CSP_DMA_XFER_PER_REQ_TMR_00_MATCH_0);

                                                                /* Configure Timer #0 for the DMA request signal.       */
    CSP_TmrOutCmpCfg(CSP_TMR_NBR_00,
                     CSP_TMR_PIN_OUT_NBR_00,                    /* ... Timer 0.                                         */
                     CSP_TMR_OPT_PIN_OUT_TOGGLE,                /* ... Match 0.                                         */
                     1u);                                       /* ... Toggle external match pin.                       */

                                                                /* Register the external interrupt 0 handler.           */
    CSP_IntVectReg((CSP_DEV_NBR   )CSP_INT_CTRL_NBR_MAIN,
                   (CSP_DEV_NBR   )CSP_INT_SRC_NBR_EINT_00,
                   (CPU_FNCT_PTR  )App_Ext0_IntHandler,
                   (void         *)ch_nbr);

    time_ctr               = 0;                                 /* Initialize the time counter and software LEDs        */
    App_ProbeLED_PwrMode   = DEF_ON;
    App_ProbeLED_HeartBeat = DEF_OFF;










    while (DEF_TRUE) {                                          /* Task body, always written as an infinite loop.       */
        OSTimeDlyHMSM(0u, 0u, 0u, 100u,                         /* Delay for 100ms.                                     */
                      OS_OPT_TIME_HMSM_STRICT,
                      &err);
        BSP_LED_Toggle(APP_LED_HEART_BEAT);                     /* Toggle Heartbeat LED.                                */

        time_ctr++;                                             /* Increment time counter.                              */

        App_ProbeLED_HeartBeat = ~App_ProbeLED_HeartBeat;       /* Toggle software LEDs                                 */

        if (time_ctr == 100u) {                                 /* Elapsed time == 10 sec?                              */
             time_ctr               = 0u;                       /* Reset time counter.                                  */
             App_ProbeLED_PwrMode   = DEF_OFF;
             App_ProbeLED_HeartBeat = DEF_OFF;

             BSP_LED_Off(APP_LED_HEART_BEAT);
             BSP_LED_Off(APP_LED_CPU_PWR_MODE);                 /* Turn off the  CPU power mode LED (SLEEP MODE).       */

                                                                /* Start DMA transfer.                                  */
             CSP_DMA_XferStartExt((CSP_DEV_NBR    ) ch_nbr,
                                  (void          *) APP_ADDR_FIO2PIN_PIN,
                                  (void          *)&App_DMA_Mem[0],
                                  (CPU_SIZE_T     ) sizeof(App_DMA_Mem),
                                  (CSP_OPT_FLAGS  ) CSP_DMA_OPT_FLAG_XFER_SRC_INC);

             CSP_TmrStart(CSP_TMR_NBR_00);                      /* Initialize Timer0                                    */

             CPU_IntSrcDis(CPU_INT_SYSTICK);                    /* Disable the SYSTICK interrupt.                       */

             CSP_IntEn(CSP_INT_CTRL_NBR_MAIN,                   /* Enable external Interrupt.                           */
                       CSP_INT_SRC_NBR_EINT_00);

             CPU_WaitForInt();                                  /* Set the CPU in sleep mode.                           */
        }
    }
}


/*
*********************************************************************************************************
*                                          App_Ext0_IntHandler()
*
* Description : External Interrupt 0 ISR.
*
* Argument(s) : p_arg    Is the argument passed to 'App_Ext0_IntHandler()' by 'CSP_IntVectReg()'
*
* Return(s)   : none
*
* Caller(s)   : This is an ISR.
*
* Note(s)     : (1) the ISR argument is used to de-allocate the DMA channel.
*********************************************************************************************************
*/

static  void  App_Ext0_IntHandler (void *p_arg)
{
    App_ProbeLED_PwrMode = DEF_ON;

    BSP_LED_Off(0);
    BSP_LED_On(APP_LED_CPU_PWR_MODE);                           /* Turn On CPU powe mode LED (NORMAL MODE).             */

    CSP_TmrStop(CSP_TMR_NBR_00);                                /* Stop the timer.                                      */

    CSP_GPIO_IntClr(CSP_GPIO_PORT_NBR_02);                      /* Clear external interrupt on Port 2.                  */

    CSP_IntDis(CSP_INT_CTRL_NBR_MAIN,                           /* Disable external Interrupt.                          */
               CSP_INT_SRC_NBR_EINT_00);

    CPU_IntSrcEn(CPU_INT_SYSTICK);                              /* Enable the SYSTICK interrupt. Resume OS operation.   */
}



/*
*********************************************************************************************************
*                                          AppTaskCreate()
*
* Description : Create application tasks.
*
* Argument(s) : none
*
* Return(s)   : none
*
* Caller(s)   : AppTaskStart()
*
* Note(s)     : none.
*********************************************************************************************************
*/

static  void  App_TaskCreate (void)
{

}


/*
*********************************************************************************************************
*                                          App_ObjCreate()
*
* Description : Create application kernel objects tasks.
*
* Argument(s) : none
*
* Return(s)   : none
*
* Caller(s)   : AppTaskStart()
*
* Note(s)     : none.
*********************************************************************************************************
*/

static  void  App_ObjCreate (void)
{
}


/*
*********************************************************************************************************
*                                          App_ObjAlloc()
*
* Description : Allocate objects form the memory heap.
*
* Argument(s) : none
*
* Return(s)   : none
*
* Caller(s)   : main()
*
* Note(s)     : none.
*********************************************************************************************************
*/

static  void  App_ObjAlloc (void)
{
    CPU_SIZE_T  octects_reqd;
    LIB_ERR     err;


    App_TaskStartTCBPtr = (OS_TCB  *)Mem_HeapAlloc((CPU_SIZE_T  ) sizeof(OS_TCB),
                                                   (CPU_SIZE_T  ) sizeof(CPU_ALIGN),
                                                   (CPU_SIZE_T *)&octects_reqd,
                                                   (LIB_ERR    *)&err);

    App_TaskStartStkPtr = (CPU_STK *)Mem_HeapAlloc((CPU_SIZE_T  ) sizeof(CPU_STK) * APP_CFG_TASK_START_STK_SIZE,
                                                   (CPU_SIZE_T  ) sizeof(CPU_STK),
                                                   (CPU_SIZE_T *)&octects_reqd,
                                                   (LIB_ERR    *)&err);
}
