/*
*********************************************************************************************************
*                                     MICRIUM BOARD SUPPORT SUPPORT
*
*                          (c) Copyright 2003-2009; Micrium, Inc.; Weston, FL
*
*               All rights reserved.  Protected by international copyright laws.
*               Knowledge of the source code may NOT be used to develop a similar product.
*               Please help us continue to provide the Embedded community with the finest
*               software available.  Your honesty is greatly appreciated.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                      BOARD SUPPORT PACKAGE (BSP)
*
*                                     KEIL MCB1700 DEVELOPMENT KIT                                    
*
* Filename      : bsp.h
* Version       : V1.00
* Programmer(s) : FT
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                                 MODULE
*
* Note(s) : (1) This header file is protected from multiple pre-processor inclusion through use of the
*               BSP present pre-processor macro definition.
*********************************************************************************************************
*/

#ifndef  BSP_PRESENT
#define  BSP_PRESENT


/*
*********************************************************************************************************
*                                            INCLUDE FILES
*********************************************************************************************************
*/

#include  <cpu.h>
#include  <cpu_core.h>

#include  <lib_def.h>
#include  <lib_ascii.h>

#include  <csp.h>


/*
*********************************************************************************************************
*                                                 EXTERNS
*********************************************************************************************************
*/


#ifdef   BSP_MODULE
#define  BSP_EXT
#else
#define  BSP_EXT  extern
#endif


/*
*********************************************************************************************************
*                                        DEFAULT CONFIGURATION
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                      JOYSTICK POSITION DEFINES
*********************************************************************************************************
*/

#define  BSP_JOY_POS_NONE                                 0u
#define  BSP_JOY_POS_UP                                   1u
#define  BSP_JOY_POS_DOWN                                 2u
#define  BSP_JOY_POS_LEFT                                 3u
#define  BSP_JOY_POS_RIGHT                                4u
#define  BSP_JOY_POS_CENTER                               5u


/*
*********************************************************************************************************
*                                          GLOBAL VARIABLES
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                         FUNCTION PROTOTYPES
*********************************************************************************************************
*/

void         BSP_PreInit            (void);
void         BSP_PostInit           (void);
void         BSP_LowLevelInit       (void);


/*
*********************************************************************************************************
*                                             LED SERVICES
*********************************************************************************************************
*/

void         BSP_LED_On             (CPU_INT08U   led);
void         BSP_LED_Off            (CPU_INT08U   led);
void         BSP_LED_Toggle         (CPU_INT08U   led);


/*
*********************************************************************************************************
*                                         PUSH BUTTON SERVICES
*********************************************************************************************************
*/

CPU_BOOLEAN  BSP_PB_StatusGet      (CPU_INT08U   pb);


/*
*********************************************************************************************************
*                                         SD CARD SERVICES
*********************************************************************************************************
*/

CPU_BOOLEAN  BSP_SD_StatusGet      (void);

/*
*********************************************************************************************************
*                                         POTENTIOMETER SERVICES
*********************************************************************************************************
*/

CPU_INT16U   BSP_PotGet             (void);


/*
*********************************************************************************************************
*                                           JOYSTICK SERVICES
*********************************************************************************************************
*/

CPU_INT08U   BSP_JoyPosGet          (void);
CPU_BOOLEAN  BSP_JoyStatGet         (CPU_INT08U  pos);


/*
*********************************************************************************************************
*                                          CONFIGURATION ERRORS
*********************************************************************************************************
*/


/*
*********************************************************************************************************
*                                              MODULE END
*********************************************************************************************************
*/

#endif    
